import { app } from '@/app.ts';
import { ConfigModel } from './models/model.ts';
import { ShareConfigService } from './services/share.ts';
app
  .route({
    path: 'config',
    key: 'list',
    middleware: ['auth'],
  })
  .define(async (ctx) => {
    const { id } = ctx.state.tokenUser;
    const config = await ConfigModel.findAll({
      where: {
        uid: id,
      },
      order: [['updatedAt', 'DESC']],
    });
    ctx.body = {
      list: config,
    };
  })
  .addTo(app);

app
  .route({
    path: 'config',
    key: 'update',
    middleware: ['auth'],
  })
  .define(async (ctx) => {
    const tokernUser = ctx.state.tokenUser;
    const tuid = tokernUser.id;
    const { id, data, ...rest } = ctx.query?.data || {};
    if (id) {
      const config = await ConfigModel.findByPk(id);
      let keyIsChange = false;
      if (rest?.key) {
        keyIsChange = rest.key !== config?.key;
      }
      if (config && config.uid === tuid) {
        if (keyIsChange) {
          const key = rest.key;
          const keyConfig = await ConfigModel.findOne({
            where: {
              key,
              uid: tuid,
            },
          });
          if (keyConfig && keyConfig.id !== id) {
            ctx.throw(403, 'key is already exists');
          }
        }
        await config.update({
          data: {
            ...config.data,
            ...data,
          },
          ...rest,
        });
        if (config.data?.permission?.share === 'public') {
          await ShareConfigService.expireShareConfig(config.key, tokernUser.username);
        }
        ctx.body = config;
      } else {
        ctx.throw(403, 'no permission');
      }
    } else if (rest?.key) {
      // id 不存在，key存在，则属于更新，key不能重复
      const key = rest.key;
      const keyConfig = await ConfigModel.findOne({
        where: {
          key,
          uid: tuid,
        },
      });
      if (keyConfig) {
        await keyConfig.update({
          data: { ...keyConfig.data, ...data },
          ...rest,
        });
        ctx.body = keyConfig;
      } else {
        const config = await ConfigModel.create({
          key,
          ...rest,
          data: data,
          uid: tuid,
        });
        ctx.body = config;
      }
    }
    if (id || rest?.key) return;

    // id和key不存在。创建一个新的配置
    const config = await ConfigModel.create({
      ...rest,
      data: data,
      uid: tuid,
    });
    ctx.body = config;
  })
  .addTo(app);

app
  .route({
    path: 'config',
    key: 'get',
    middleware: ['auth'],
  })
  .define(async (ctx) => {
    const tokernUser = ctx.state.tokenUser;
    const tuid = tokernUser.id;
    const { id, key } = ctx.query?.data || {};
    if (!id && !key) {
      ctx.throw(400, 'id or key is required');
    }
    let config: ConfigModel;
    if (id) {
      config = await ConfigModel.findByPk(id);
    }
    if (!config && key) {
      config = await ConfigModel.findOne({
        where: {
          key,
          uid: tuid,
        },
      });
    }
    if (!config) {
      ctx.throw(404, 'config not found');
    }
    if (config && config.uid === tuid) {
      ctx.body = config;
    } else {
      ctx.throw(403, 'no permission');
    }
  })
  .addTo(app);

app
  .route({
    path: 'config',
    key: 'delete',
    middleware: ['auth'],
  })
  .define(async (ctx) => {
    const tokernUser = ctx.state.tokenUser;
    const tuid = tokernUser.id;
    const { id } = ctx.query?.data || {};
    if (id) {
      const config = await ConfigModel.findOne({
        where: {
          id,
        },
      });
      if (config && config.uid === tuid) {
        await config.destroy();
      } else {
        ctx.throw(403, 'no permission');
      }
    } else {
      ctx.throw(400, 'id is required');
    }
  })
  .addTo(app);
